@@ -232,6 +232,7 @@ class ControlHostBootstrap(KayobeAnsibleMixin, KollaAnsibleMixin, VaultMixin,
     def take_action(self, parsed_args):
         self.app.LOG.debug("Bootstrapping Kayobe Ansible control host")
         ansible.install_galaxy_roles(parsed_args)
+        ansible.install_galaxy_collections(parsed_args)
         playbooks = _build_playbook_list("bootstrap")
         self.run_kayobe_playbooks(parsed_args, playbooks, ignore_limit=True)
 
@@ -271,8 +272,9 @@ class ControlHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
         # Remove roles that are no longer used. Do this before installing new
         # ones, just in case a custom role dependency includes any.
         ansible.prune_galaxy_roles(parsed_args)
-        # Use force to upgrade roles.
+        # Use force to upgrade roles and collections.
         ansible.install_galaxy_roles(parsed_args, force=True)
+        ansible.install_galaxy_collections(parsed_args, force=True)
         playbooks = _build_playbook_list("bootstrap")
         self.run_kayobe_playbooks(parsed_args, playbooks, ignore_limit=True)
         playbooks = _build_playbook_list("kolla-ansible")
@@ -413,6 +415,8 @@ class SeedHypervisorHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
     * Optionally, wipe unmounted disk partitions (--wipe-disks).
     * Configure user accounts, group associations, and authorised SSH keys.
     * Configure the host's network interfaces.
+    * Configure a firewall.
+    * Configure tuned profile.
     * Set sysctl parameters.
     * Configure timezone and ntp.
     * Optionally, configure software RAID arrays.
@@ -443,8 +447,8 @@ class SeedHypervisorHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin,
         if parsed_args.wipe_disks:
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
-            "users", "dev-tools", "network", "sysctl", "time",
-            "mdadm", "luks", "lvm", "seed-hypervisor-libvirt-host")
+            "users", "dev-tools", "network", "firewall", "tuned", "sysctl",
+            "time", "mdadm", "luks", "lvm", "seed-hypervisor-libvirt-host")
         self.run_kayobe_playbooks(parsed_args, playbooks,
                                   limit="seed-hypervisor")
 
@@ -561,6 +565,8 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     * Configure user accounts, group associations, and authorised SSH keys.
     * Disable SELinux.
     * Configure the host's network interfaces.
+    * Configure a firewall.
+    * Configure tuned profile.
     * Set sysctl parameters.
     * Configure IP routing and source NAT.
     * Disable bootstrap interface configuration.
@@ -597,8 +603,8 @@ class SeedHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         if parsed_args.wipe_disks:
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
-            "users", "dev-tools", "disable-selinux", "network",
-            "sysctl", "ip-routing", "snat", "disable-glean", "time",
+            "users", "dev-tools", "disable-selinux", "network", "firewall",
+            "tuned", "sysctl", "ip-routing", "snat", "disable-glean", "time",
             "mdadm", "luks", "lvm", "docker-devicemapper",
             "kolla-ansible-user", "kolla-pip", "kolla-target-venv")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="seed")
@@ -811,6 +817,173 @@ class SeedDeploymentImageBuild(KayobeAnsibleMixin, VaultMixin, Command):
                                   extra_vars=extra_vars)
 
 
+class InfraVMProvision(KayobeAnsibleMixin, VaultMixin, Command):
+    """Provisions infra virtual machines
+
+    * Allocate IP addresses for all configured networks.
+    * Provision a virtual machine using libvirt.
+    """
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Provisioning infra VMs")
+        self.run_kayobe_playbook(parsed_args,
+                                 _get_playbook_path("ip-allocation"),
+                                 limit="infra-vms")
+
+        limit_arg = utils.intersect_limits(parsed_args.limit, "infra-vms")
+        # We want the limit to affect one play only. To do this we use a
+        # variable to override the hosts list instead of using limit.
+        extra_vars = {
+            "infra_vm_limit": limit_arg
+        }
+
+        self.run_kayobe_playbook(parsed_args,
+                                 _get_playbook_path("infra-vm-provision"),
+                                 ignore_limit=True, extra_vars=extra_vars)
+
+
+class InfraVMDeprovision(KayobeAnsibleMixin, VaultMixin, Command):
+    """Deprovisions infra virtual machines.
+
+    This will destroy all infra VMs and all associated volumes.
+    """
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Deprovisioning infra VMs")
+        # We want the limit to affect one play only. To do this we use a
+        # variable to override the hosts list instead of using limit.
+        limit_arg = utils.intersect_limits(parsed_args.limit, "infra-vms")
+        extra_vars = {
+            "infra_vm_limit": limit_arg
+        }
+
+        self.run_kayobe_playbook(parsed_args,
+                                 _get_playbook_path("infra-vm-deprovision"),
+                                 ignore_limit=True, extra_vars=extra_vars)
+
+
+class InfraVMHostConfigure(KayobeAnsibleMixin, VaultMixin,
+                           Command):
+    """Configure the infra VMs host OS and services.
+
+    * Allocate IP addresses for all configured networks.
+    * Add the host to SSH known hosts.
+    * Configure a user account for use by kayobe for SSH access.
+    * Configure package repos.
+    * Configure a PyPI mirror.
+    * Optionally, create a virtualenv for remote target hosts.
+    * Optionally, wipe unmounted disk partitions (--wipe-disks).
+    * Configure user accounts, group associations, and authorised SSH keys.
+    * Disable SELinux.
+    * Configure the host's network interfaces.
+    * Configure a firewall.
+    * Configure tuned profile.
+    * Set sysctl parameters.
+    * Disable bootstrap interface configuration.
+    * Configure timezone.
+    * Optionally, configure software RAID arrays.
+    * Optionally, configure encryption.
+    * Configure LVM volumes.
+    """
+
+    def get_parser(self, prog_name):
+        parser = super(InfraVMHostConfigure, self).get_parser(prog_name)
+        group = parser.add_argument_group("Host Configuration")
+        group.add_argument("--wipe-disks", action='store_true',
+                           help="wipe partition and LVM data from all disks "
+                                "that are not mounted. Warning: this can "
+                                "result in the loss of data")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Configuring Infra VMs host OS")
+
+        # Allocate IP addresses.
+        playbooks = _build_playbook_list("ip-allocation")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="infra-vms")
+
+        # Kayobe playbooks.
+        playbooks = _build_playbook_list(
+            "ssh-known-host", "kayobe-ansible-user",
+            "apt", "dnf", "pip", "kayobe-target-venv")
+        if parsed_args.wipe_disks:
+            playbooks += _build_playbook_list("wipe-disks")
+        playbooks += _build_playbook_list(
+            "users", "dev-tools", "disable-selinux", "network", "firewall",
+            "tuned", "sysctl", "disable-glean", "disable-cloud-init", "time",
+            "mdadm", "luks", "lvm", "docker-devicemapper", "docker")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="infra-vms")
+
+
+class InfraVMHostPackageUpdate(KayobeAnsibleMixin, VaultMixin, Command):
+    """Update packages on the infra VMs."""
+
+    def get_parser(self, prog_name):
+        parser = super(InfraVMHostPackageUpdate, self).get_parser(prog_name)
+        group = parser.add_argument_group("Host Package Updates")
+        group.add_argument("--packages", required=True,
+                           help="List of packages to update. Use '*' to "
+                                "update all packages.")
+        group.add_argument("--security", action='store_true',
+                           help="Only install updates that have been marked "
+                                "security related.")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Updating infra vm host packages")
+        extra_vars = {
+            "host_package_update_packages": parsed_args.packages,
+            "host_package_update_security": parsed_args.security,
+        }
+        playbooks = _build_playbook_list("host-package-update")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="infra-vms",
+                                  extra_vars=extra_vars)
+
+
+class InfraVMHostCommandRun(KayobeAnsibleMixin, VaultMixin, Command):
+    """Run command on the infra VMs."""
+
+    def get_parser(self, prog_name):
+        parser = super(InfraVMHostCommandRun, self).get_parser(prog_name)
+        group = parser.add_argument_group("Host Command Run")
+        group.add_argument("--command", required=True,
+                           help="Command to run (required).")
+        group.add_argument("--show-output", action='store_true',
+                           help="Show command output")
+        return parser
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Run command on infra VM hosts")
+        extra_vars = {
+            "host_command_to_run": utils.escape_jinja(parsed_args.command),
+            "show_output": parsed_args.show_output}
+        playbooks = _build_playbook_list("host-command-run")
+        self.run_kayobe_playbooks(parsed_args, playbooks, limit="infra-vms",
+                                  extra_vars=extra_vars)
+
+
+class InfraVMHostUpgrade(KayobeAnsibleMixin, VaultMixin, Command):
+    """Upgrade the infra VM host services.
+
+    Performs the changes necessary to make the host services suitable for the
+    configured OpenStack release.
+    """
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Upgrading the infra-vm host services")
+        playbooks = _build_playbook_list("kayobe-target-venv")
+        self.run_kayobe_playbooks(parsed_args, playbooks,
+                                  limit="infra-vms")
+
+
+class InfraVMServiceDeploy(KayobeAnsibleMixin, VaultMixin,
+                           Command):
+    """Run hooks for infra structure services."""
+
+    def take_action(self, parsed_args):
+        self.app.LOG.debug("Running no-op Infra VM service deploy")
+
+
 class OvercloudInventoryDiscover(KayobeAnsibleMixin, VaultMixin, Command):
     """Discover the overcloud inventory from the seed's Ironic service.
 
@@ -830,8 +1003,7 @@ class OvercloudInventoryDiscover(KayobeAnsibleMixin, VaultMixin, Command):
                                      "overcloud-inventory-discover"))
         # If necessary, allocate IP addresses for the discovered hosts.
         self.run_kayobe_playbook(parsed_args,
-                                 _get_playbook_path("ip-allocation"),
-                                 limit="overcloud")
+                                 _get_playbook_path("ip-allocation"))
         # Now populate the Kolla Ansible inventory.
         self.generate_kolla_ansible_config(parsed_args, service_config=False)
 
@@ -953,6 +1125,8 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
     * Configure user accounts, group associations, and authorised SSH keys.
     * Disable SELinux.
     * Configure the host's network interfaces.
+    * Configure a firewall.
+    * Configure tuned profile.
     * Set sysctl parameters.
     * Disable bootstrap interface configuration.
     * Configure timezone and ntp.
@@ -987,8 +1161,8 @@ class OvercloudHostConfigure(KollaAnsibleMixin, KayobeAnsibleMixin, VaultMixin,
         if parsed_args.wipe_disks:
             playbooks += _build_playbook_list("wipe-disks")
         playbooks += _build_playbook_list(
-            "users", "dev-tools", "disable-selinux", "network",
-            "sysctl", "disable-glean", "disable-cloud-init", "time",
+            "users", "dev-tools", "disable-selinux", "network", "firewall",
+            "tuned", "sysctl", "disable-glean", "disable-cloud-init", "time",
             "mdadm", "luks", "lvm", "docker-devicemapper",
             "kolla-ansible-user", "kolla-pip", "kolla-target-venv")
         self.run_kayobe_playbooks(parsed_args, playbooks, limit="overcloud")
@@ -1090,10 +1264,6 @@ class OvercloudDatabaseBackup(KollaAnsibleMixin, KayobeAnsibleMixin,
 
     def take_action(self, parsed_args):
         self.app.LOG.debug("Performing overcloud database backup")
-
-        # First prepare configuration.
-        self.generate_kolla_ansible_config(parsed_args, service_config=False)
-
         extra_args = []
         if parsed_args.incremental:
             extra_args.append('--incremental')
@@ -1118,10 +1288,6 @@ class OvercloudDatabaseRecover(KollaAnsibleMixin, KayobeAnsibleMixin,
     def take_action(self, parsed_args):
         self.app.LOG.debug("Performing overcloud database recovery")
         extra_vars = {}
-
-        # First prepare configuration.
-        self.generate_kolla_ansible_config(parsed_args, service_config=True)
-
         if parsed_args.force_recovery_host:
             extra_vars['mariadb_recover_inventory_name'] = (
                 parsed_args.force_recovery_host)
