@@ -591,11 +591,6 @@ def net_configdrive_network_device(context, name, inventory_hostname=None):
     gateway = net_gateway(context, name, inventory_hostname)
     bootproto = net_bootproto(context, name, inventory_hostname)
     mtu = net_mtu(context, name, inventory_hostname)
-    vlan = net_vlan(context, name, inventory_hostname)
-    if vlan and '.' in device:
-        backend = [device.split('.')[0]]
-    else:
-        backend = None
     interface = {
         'device': device,
         'address': ip,
@@ -603,10 +598,7 @@ def net_configdrive_network_device(context, name, inventory_hostname=None):
         'gateway': gateway,
         'bootproto': bootproto or 'static',
         'mtu': mtu,
-        'backend': backend,
     }
-    if backend:
-        interface['type'] = 'vlan'
     interface = {k: v for k, v in interface.items() if v is not None}
     return interface
 
